@@ -26,103 +26,24 @@ pgfSweaveDriver <- function() {
     list(
        setup = pgfSweaveSetup,
        runcode = pgfSweaveRuncode,
-       writedoc = utils::RweaveLatexWritedoc,
+       writedoc = pgfSweaveWritedoc,
        finish = utils::RweaveLatexFinish,
-       checkopts = utils::RweaveLatexOptions
+       checkopts = pgfSweaveOptions 
        )
 }
 
     
 source('R/cacheSweaveUnexportedFunctions.R')
+source('R/utilities.R')
 
-
-################################################################################
-## [RDP]
-## The major modification is here: Rather than evaluate expressions
-## and leave them in the global environment, we evaluate them in a
-## local environment (that has globalenv() as the parent) and then
-## store the assignments in a 'stashR' database.  If an expression
-## does not give rise to new R objects, then nothing is saved.
-##
-## For each expression ('expr'), we compute a digest and associate
-## with that digest the names of the objects that were created by
-## evaluating the expression.  That way, for a given cached
-## expression, we know which keys to lazy-load from the cache when
-## evaluation is skipped.
-## end [RDP]
-##
-## [CWB]
-## minor modification to the original function 'cacheSweaveEvalWithOpt' has 
-## two outputs helping to improve the recognition of changes to code chunks
-## end [CWB]
-################################################################################
-
-
-pgfSweaveEvalWithOpt <- function (expr, options) {
-  chunkDigest <- options$chunkDigest
-  
-  ## 'expr' is a single expression, so something like 'a <- 1'
-  res <- NULL
-  chunkChanged <- TRUE
-
-  if(!options$eval)
-      return(res)
-  if(options$cache) {
-    cachedir <- getCacheDir()
-  
-    ## Create database name from chunk label and MD5
-    ## digest
-    dbName <- makeChunkDatabaseName(cachedir, options,chunkDigest)
-    exprDigest <- mangleDigest(digest(expr))
-  
-    ## Create 'stashR' database
-    db <- new("localDB", dir = dbName, name = basename(dbName))
-  
-    ## If the current expression is not cached, then
-    ## evaluate the expression and dump the resulting
-    ## objects to the database.  Otherwise, just read the
-    ## vector of keys from the database
-  
-    if(!dbExists(db, exprDigest)){
-      keys <- try({
-        evalAndDumpToDB(db, expr, exprDigest)
-        }, silent = TRUE)
-    }
-    else{
-      keys <- dbFetch(db, exprDigest)
-      chunkChanged <- FALSE
-    }
-    
-  
-    ## If there was an error then just return the
-    ## condition object and let Sweave deal with it.
-    if(inherits(keys, "try-error"))
-      return(list(err=keys,chunkChanged=chunkChanged))
-  
-    dbLazyLoad(db, globalenv(), keys)
-  }
-  else {
-      ## If caching is turned off, just evaluate the expression
-      ## in the global environment
-      res <- try(.Internal(eval.with.vis(expr, .GlobalEnv,baseenv())),silent=TRUE)
-      
-      if(inherits(res, "try-error"))
-        return(list(err=res,chunkChanged=chunkChanged))
-      if(options$print | (options$term & res$visible))
-        print(res$value)
-  }
-  list(err=res,chunkChanged=chunkChanged)
-}
-
-## Add the 'pgf' and 'external' and 'pdflatex' option to the list
+## Add the 'pgf' and 'external', 'pdflatex', 'sanitize' option to the list
 pgfSweaveSetup <- function(file, syntax,
               output=NULL, quiet=FALSE, debug=FALSE, echo=TRUE,
               eval=TRUE, split=FALSE, stylepath=TRUE, 
               pdf=FALSE, eps=FALSE, cache=FALSE, pgf=FALSE, 
-              tikz=TRUE, external=FALSE, sanitize = FALSE,
-              tex.driver="pdflatex")
+              tikz=TRUE, external=FALSE, sanitize = FALSE, 
+              highlight = FALSE, tidy = FALSE, tex.driver="pdflatex")
 {
-
     out <- utils::RweaveLatexSetup(file, syntax, output=output, quiet=quiet,
                      debug=debug, echo=echo, eval=eval,
                      split=split, stylepath=stylepath, pdf=pdf,
@@ -137,8 +58,12 @@ pgfSweaveSetup <- function(file, syntax,
     out$options[["pgf"]] <- pgf
     out$options[["tikz"]] <- tikz
     out$options[["external"]] <- external
-    out[["tex.driver"]] <- tex.driver
+    out$options[["tex.driver"]] <- tex.driver
     out$options[["sanitize"]] <- sanitize
+    out$options[["highlight"]] <- ifelse(echo,highlight,FALSE)
+    out$options[["tidy"]] <- tidy
+    out[["haveHighlightSyntaxDef"]] <- FALSE
+    out[["haveRealjobname"]] <- FALSE
     ## end [CWB]
 
     ## We assume that each .Rnw file gets its own map file
@@ -150,93 +75,164 @@ pgfSweaveSetup <- function(file, syntax,
     ##      graphic 
     out[["shellFile"]] <- makeExternalShellScriptName(file)
     out[["srcfileName"]] <- sub("\\.Rnw$", "\\.tex", file)
+    out[["jobname"]] <- basename(removeExt(file))
     file.create(out[["shellFile"]])  ## Overwrite an existing file
     ######################################################################
 
     out
 }
 
-makeExternalShellScriptName <- function(Rnwfile) {
-    shellfile <- sub("\\.Rnw$", "\\.sh", Rnwfile)
+    # changes in Sweave in 2.12 make it necessary to copy this function here 
+    # to register the tex.driver option, I wish there was an easier way....
+pgfSweaveOptions <- function(options)
+{
 
-    ## Don''t clobber
-    if(identical(shellfile, Rnwfile))
-        shellfile <- paste(Rnwfile, "sh", sep = ".")
-    shellfile
+    ## ATTENTION: Changes in this function have to be reflected in the
+    ## defaults in the init function!
+
+    ## convert a character string to logical
+    c2l <- function(x){
+        if(is.null(x)) return(FALSE)
+        else return(as.logical(toupper(as.character(x))))
+    }
+
+    NUMOPTS <- c("width", "height")
+    NOLOGOPTS <- c(NUMOPTS, "results", "prefix.string",
+                   "engine", "label", "strip.white",
+                   "pdf.version", "pdf.encoding", "tex.driver")
+    for(opt in names(options)){
+        if(! (opt %in% NOLOGOPTS)){
+            oldval <- options[[opt]]
+            if(!is.logical(options[[opt]])){
+                options[[opt]] <- c2l(options[[opt]])
+            }
+            if(is.na(options[[opt]]))
+                stop(gettextf("invalid value for '%s' : %s", opt, oldval),
+                     domain = NA)
+        }
+        else if(opt %in% NUMOPTS){
+            options[[opt]] <- as.numeric(options[[opt]])
+        }
+    }
+
+    if(!is.null(options$results))
+        options$results <- tolower(as.character(options$results))
+    options$results <- match.arg(options$results,
+                                 c("verbatim", "tex", "hide"))
+
+    if(!is.null(options$strip.white))
+        options$strip.white <- tolower(as.character(options$strip.white))
+    options$strip.white <- match.arg(options$strip.white,
+                                     c("true", "false", "all"))
+    options
 }
 
-## preserve comments in the R code and at the same time make them 'well-formatted'
-tidy.source = function(source = "clipboard", keep.comment = TRUE,
-    keep.blank.line = TRUE, begin.comment, end.comment, output = TRUE,
-    width.cutoff = 60L, ...) {
-    if (source == "clipboard" && Sys.info()["sysname"] == "Darwin") {
-        source = pipe("pbpaste")
-    }
-    tidy.block = function(block.text) {
-        exprs = base::parse(text = block.text)
-        n = length(exprs)
-        res = character(n)
-        for (i in 1:n) {
-            dep = paste(base::deparse(exprs[i], width.cutoff),
-                collapse = "\n")
-            res[i] = substring(dep, 12, nchar(dep) - 1)
-        }
-        return(res)
-    }
-    text.lines = readLines(source, warn = FALSE)
-    if (keep.comment) {
-        identifier = function() paste(sample(LETTERS), collapse = "")
-        if (missing(begin.comment))
-            begin.comment = identifier()
-        if (missing(end.comment))
-            end.comment = identifier()
-        text.lines = gsub("^[[:space:]]+|[[:space:]]+$", "",
-            text.lines)
-        while (length(grep(sprintf("%s|%s", begin.comment, end.comment),
-            text.lines))) {
-            begin.comment = identifier()
-            end.comment = identifier()
-        }
-        head.comment = substring(text.lines, 1, 1) == "#"
-        if (any(head.comment)) {
-            text.lines[head.comment] = gsub("\"", "'", text.lines[head.comment])
-            text.lines[head.comment] = sprintf("%s=\"%s%s\"",
-                begin.comment, text.lines[head.comment], end.comment)
-        }
-        blank.line = text.lines == ""
-        if (any(blank.line) & keep.blank.line)
-            text.lines[blank.line] = sprintf("%s=\"%s\"", begin.comment,
-                end.comment)
-        text.mask = tidy.block(text.lines)
-        text.tidy = gsub(sprintf("%s = \"|%s\"", begin.comment,
-            end.comment), "", text.mask)
-    }
-    else {
-        text.tidy = text.mask = tidy.block(text.lines)
-        begin.comment = end.comment = ""
-    }
-    if (output)
-        cat(paste(text.tidy, collapse = "\n"), "\n", ...)
-    invisible(list(text.tidy = text.tidy, text.mask = text.mask,
-        begin.comment = begin.comment, end.comment = end.comment))
-}
+    # Copied from Sweave in R version 2.12, internal chages make it necessary
+    # to register the tex.driver option as a "NOLOGOPT"
+pgfSweaveWritedoc <- function(object, chunk)
+{
+    linesout <- attr(chunk, "srclines")
 
-## to replace the default parse()
-parse2 = function(text, ...) {
-    zz = tempfile()
-    enc = options(encoding = "native.enc")
-    writeLines(text, zz)
-    tidy.res = tidy.source(zz, out = FALSE, keep.blank.line = FALSE)
-    options(enc)
-    unlink(zz)
-    options(begin.comment = tidy.res$begin.comment, end.comment = tidy.res$end.comment)
-    base::parse(text = tidy.res$text.mask)
-}
+    if(length(grep("\\usepackage[^\\}]*Sweave.*\\}", chunk)))
+        object$havesty <- TRUE
+        
+    haverealjobname <- FALSE
+    if(length(grep("\\pgfrealjobname\\{.*\\}", chunk)))
+        haverealjobname <- TRUE
+        
+    
+    if(!object$havesty){
+    begindoc <- "^[[:space:]]*\\\\begin\\{document\\}"
+    which <- grep(begindoc, chunk)
+    if (length(which)) {
+      print(object$srcfile)
+            chunk[which] <- paste("\\usepackage{", object$styfile, "}\n",
+                  chunk[which], sep="")
+            linesout <- linesout[c(1L:which, which, seq(from=which+1L, length.out=length(linesout)-which))]
+            object$havesty <- TRUE
+        }
+    }
+    
+      # add pgfrealjobname if it doesnt exist
+    if(!haverealjobname){
+     begindoc <- "^[[:space:]]*\\\\begin\\{document\\}"
+     which <- grep(begindoc, chunk)
+      # if Sweave line also does not exist
+     otherwhich <- grep("\\usepackage[^\\}]*Sweave.*\\}", chunk)
+     if(length(which) == 0) which <- otherwhich
+     if (length(which)) {
+             chunk[which] <- paste("\\pgfrealjobname{",object$jobname,"}\n",
+                      chunk[which], sep="")
+             linesout <- linesout[c(1L:which, which, seq(from=which+1L, length.out=length(linesout)-which))]
+             object$haverealjobname <- TRUE
+         }
+     }
+     
+     # always add the syntax definitions because there is no real way to 
+     # check if a single code chunk as the option before hand. 
+      if (!object$haveHighlightSyntaxDef){
+                # get the latex style definitions from the highlight package
+          tf <- tempfile()
+          cat(styler('default', 'sty', styler_assistant_latex),sep='\n',file=tf)
+          cat(boxes_latex(),sep='\n',file=tf,append=T)
+          hstyle <- readLines(tf)
+                # find where to put the style definitions
+        begindoc <- "^[[:space:]]*\\\\begin\\{document\\}"
+            which <- grep(begindoc, chunk)
+            otherwhich <- grep("\\usepackage[^\\}]*Sweave.*\\}", chunk)
+            if(!length(which)) which <- otherwhich
+                # put in the style definitions before the \begin{document}
+        if(length(which)) {
+                chunk <- c(chunk[1:(which-1)],hstyle,chunk[which:length(chunk)])
 
-## to replace the default deparse()
-deparse2 = function(expr, ...) {
-    gsub(sprintf("%s = \"|%s\"", getOption("begin.comment"),
-        getOption("end.comment")), "", base::deparse(expr, ...))
+                linesout <- linesout[c(1L:which, which, seq(from = which + 
+                    1L, length.out = length(linesout) - which))]
+          object$haveHighlightSyntaxDef <- TRUE
+        }
+      }
+     
+    while(length(pos <- grep(object$syntax$docexpr, chunk)))
+    {
+        cmdloc <- regexpr(object$syntax$docexpr, chunk[pos[1L]])
+        cmd <- substr(chunk[pos[1L]], cmdloc,
+                      cmdloc+attr(cmdloc, "match.length")-1L)
+        cmd <- sub(object$syntax$docexpr, "\\1", cmd)
+        if(object$options$eval){
+            val <- as.character(eval(parse(text=cmd), envir=.GlobalEnv))
+            ## protect against character(0L), because sub() will fail
+            if(length(val) == 0L) val <- ""
+        }
+        else
+            val <- paste("\\\\verb{<<", cmd, ">>{", sep="")
+
+        chunk[pos[1L]] <- sub(object$syntax$docexpr, val, chunk[pos[1L]])
+    }
+    while(length(pos <- grep(object$syntax$docopt, chunk)))
+    {
+        opts <- sub(paste(".*", object$syntax$docopt, ".*", sep=""),
+                    "\\1", chunk[pos[1L]])
+        object$options <- utils:::SweaveParseOptions(opts, object$options,
+                                             pgfSweaveOptions)
+                                              browser()
+        if (isTRUE(object$options$concordance)
+              && !object$haveconcordance) {
+            savelabel <- object$options$label
+            object$options$label <- "concordance"
+            prefix <- utils:::RweaveChunkPrefix(object$options)
+            object$options$label <- savelabel
+            object$concordfile <- paste(prefix, "tex", sep=".")
+            chunk[pos[1L]] <- sub(object$syntax$docopt,
+                                 paste("\\\\input{", prefix, "}", sep=""),
+                                 chunk[pos[1L]])
+            object$haveconcordance <- TRUE
+        } else
+            chunk[pos[1L]] <- sub(object$syntax$docopt, "", chunk[pos[1L]])
+    }
+
+    cat(chunk, sep="\n", file=object$output, append=TRUE)
+    object$linesout <- c(object$linesout, linesout)
+
+    return(object)
 }
 
 
@@ -255,7 +251,11 @@ pgfSweaveRuncode <- function(object, chunk, options) {
 
   if(!object$quiet){
     cat(formatC(options$chunknr, width=2), ":")
-    if(options$echo) cat(" echo")
+    if(options$echo){
+      cat(" echo")
+      if(options$highlight) cat(" highlight")
+      if(options$tidy) cat(" tidy")
+    }
     if(options$keep.source) cat(" keep.source")
     if(options$eval){
       if(options$print) cat(" print")
@@ -275,6 +275,7 @@ pgfSweaveRuncode <- function(object, chunk, options) {
       cat("\n")
   }
 
+
   chunkprefix <- RweaveChunkPrefix(options)
 
   if(options$split){
@@ -292,9 +293,16 @@ pgfSweaveRuncode <- function(object, chunk, options) {
   on.exit(options(saveopts))
 
   SweaveHooks(options, run=TRUE)
-
+  # remove unwanted "#line" comments added by R 2.12
+  if(substring(chunk[1], 1, 5) == "#line") chunk <- chunk[-1]
+  
   ## parse entire chunk block
-  chunkexps <- try(parse2(text=chunk), silent=TRUE)
+  chunkexps <- 
+    if(options$tidy){
+      try(parse2(text=chunk), silent=TRUE) 
+    }else{
+      try(parse(text=chunk), silent=TRUE)
+    }
   RweaveTryStop(chunkexps, options)
 
   ## [CWB] Create a DB entry which is simply the digest of the text of 
@@ -306,9 +314,9 @@ pgfSweaveRuncode <- function(object, chunk, options) {
   ## will get regenerated.
   chunkTextEvalString <- paste("chunkText <- '", 
     digest(paste(chunk,collapse='')), "'", sep='')
-  #print(chunkTextEvalString)
-  if(substr(chunk[1],1,9)!='chunkText')
-    chunk <- c(chunkTextEvalString, chunk)
+  attr(chunk, "digest") <- digest(paste(chunk,collapse=''))
+  #if(substr(chunk[1],1,9)!='chunkText')
+  #  chunk <- c(chunkTextEvalString, chunk)
   ## end [CWB]
 
   ## Adding my own stuff here [RDP]
@@ -332,10 +340,10 @@ pgfSweaveRuncode <- function(object, chunk, options) {
   else
     lastshown <- srcline - 1
   thisline <- 0
+  
   for(nce in 1:length(chunkexps)){
     ce <- chunkexps[[nce]]
-    if (nce <= length(srcrefs) && 
-      !is.null(srcref <- srcrefs[[nce]])) {
+    if (nce <= length(srcrefs) && !is.null(srcref <- srcrefs[[nce]])) {
         if (options$expand) {
           srcfile <- attr(srcref, "srcfile")
           showfrom <- srcref[1]
@@ -346,22 +354,34 @@ pgfSweaveRuncode <- function(object, chunk, options) {
           showto <- srclines[srcref[3]]
         }
         dce <- getSrcLines(srcfile, lastshown+1, showto)
+            # replace the comment identifiers
+        if(options$tidy){
+          dce <- gsub(sprintf("%s = \"|%s\"", getOption("begin.comment"),
+              getOption("end.comment")), "", dce)
+              # replace tabs with spaces for better looking output
+          dce <- gsub("\\\\t", "    ", dce)
+        }
+            # replace leading lines with #line from 2.12.0
         leading <- showfrom-lastshown
         lastshown <- showto
         srcline <- srclines[srcref[3]]
-        while (length(dce) 
-          && length(grep("^[ \\t]*$", dce[1]))) {
+        while (length(dce) && length(grep("^[ \\t]*$", dce[1]))) {
           dce <- dce[-1]
           leading <- leading - 1
         }
     } else {
-      dce <- deparse2(ce, width.cutoff = 0.75*getOption("width"))
+      dce <- 
+      if(options$tidy){
+        deparse2(ce, width.cutoff = 0.75*getOption("width"))
+      }else{
+        deparse(ce, width.cutoff = 0.75*getOption("width"))
+      }
       leading <- 1
     }
     if(object$debug)
       cat("\nRnw> ", paste(dce, collapse="\n+  "),"\n")
     if(options$echo && length(dce)){
-      if(!openSinput){
+      if(!openSinput & !options$highlight){
         if(!openSchunk){
           cat("\\begin{Schunk}\n",file=chunkout, append=TRUE)
             linesout[thisline + 1] <- srcline
@@ -371,11 +391,32 @@ pgfSweaveRuncode <- function(object, chunk, options) {
           cat("\\begin{Sinput}",file=chunkout, append=TRUE)
           openSinput <- TRUE
       }
-      cat("\n",paste(getOption("prompt"), dce[1:leading], sep="", 
-        collapse="\n"), file=chunkout, append=TRUE, sep="")
-      if (length(dce) > leading)
-        cat("\n", paste(getOption("continue"), dce[-(1:leading)], sep="", 
+
+         # Code highlighting stuff
+      if(options$highlight){
+  
+        if(length(grep("^[[:space:]]*$",dce)) >= 1){
+            # for blank lines for which parser throws an error 
+          cat(translator_latex(paste(getOption("prompt"),'\n', sep="")), 
+            file=chunkout, append=TRUE, sep="")
+          cat(newline_latex(),file=chunkout, append=TRUE)
+        }else{
+          if(nce == 1)
+            cat(newline_latex(),file=chunkout, append=TRUE)
+          highlight(parser.output=parser(text=dce),
+            renderer=renderer_latex(document=FALSE),
+            output = chunkout, showPrompts=TRUE,final.newline = TRUE)
+              # highlight doesnt put in an ending newline for some reason
+          cat(newline_latex(),file=chunkout, append=TRUE)
+        }
+
+      }else{
+        cat("\n",paste(getOption("prompt"), dce[1:leading], sep="", 
           collapse="\n"), file=chunkout, append=TRUE, sep="")
+        if (length(dce) > leading)
+          cat("\n", paste(getOption("continue"), dce[-(1:leading)], sep="", 
+            collapse="\n"), file=chunkout, append=TRUE, sep="")
+      }
       linesout[thisline + 1:length(dce)] <- srcline
       thisline <- thisline + length(dce)
     }
@@ -457,6 +498,9 @@ pgfSweaveRuncode <- function(object, chunk, options) {
     }
   }
 
+  if(options$highlight)
+    cat("\n", file=chunkout, append=TRUE)
+
   if(openSinput){
     cat("\n\\end{Sinput}\n", file=chunkout, append=TRUE)
     linesout[thisline + 1:2] <- srcline
@@ -469,6 +513,10 @@ pgfSweaveRuncode <- function(object, chunk, options) {
     thisline <- thisline + 1
   }
 
+  #put in an extra newline before the output for good measure
+  cat("\n", file=chunkout, append=TRUE)
+
+
   if(is.null(options$label) & options$split)
     close(chunkout)
 
@@ -568,7 +616,8 @@ pgfSweaveRuncode <- function(object, chunk, options) {
       }
     }
     if(options$tikz){
-      if(chunkChanged | !pdfExists){
+      if(chunkChanged | ifelse(options$tex.driver == "latex", 
+          !epsExists,!pdfExists)){
         tikzDevice::tikz(file=paste(chunkprefix, "tikz", sep="."), 
           width=options$width, height=options$height, 
           sanitize=options$sanitize)
@@ -583,17 +632,24 @@ pgfSweaveRuncode <- function(object, chunk, options) {
     }
     
     if(options$external){
-      if( chunkChanged | !pdfExists && (!options$pdf && !options$eps)){
+      if( chunkChanged | ifelse(options$tex.driver == "latex", 
+          !epsExists,!pdfExists) && (!options$pdf && !options$eps)){
       
         shellFile <- object[["shellFile"]]
-        tex.driver <- object[["tex.driver"]]
+        tex.driver <- options[["tex.driver"]]
         
         cat(tex.driver, ' --jobname=', chunkprefix,' ', 
           object[["srcfileName"]], '\n', sep='',  file=shellFile, append=TRUE)
+        if(tex.driver == "latex"){
+          cat('dvipdf ',paste(chunkprefix,'dvi',sep='.'), '\n', sep='', 
+            file=shellFile, append=TRUE)
+          cat('pdftops -eps ',paste(chunkprefix,'pdf',sep='.'),
+            '\n', sep='', file=shellFile, append=TRUE)
+        }
       }
     }
     if(options$include && options$external) {
-      cat("\\beginpgfgraphicnamed{",chunkprefix,"}\n",sep="",
+      cat("\n\\beginpgfgraphicnamed{",chunkprefix,"}\n",sep="",
         file=object$output, append=TRUE)
       linesout[thisline + 1] <- srcline
       thisline <- thisline + 1
